﻿namespace Connected.Services;
public abstract class NullableServiceFunction<TArgs, TReturnValue> : ServiceOperation<TArgs>, INullableFunction<TArgs, TReturnValue>
	where TArgs : IDto
{
	protected TReturnValue? Result { get; private set; }
	public async Task<TReturnValue?> Invoke(TArgs args)
	{
		if (args is null)
			throw new ArgumentException(null, nameof(args));

		Arguments = args;

		Result = await OnInvoke();

		return Result;
	}

	protected virtual async Task<TReturnValue?> OnInvoke()
	{
		await Task.CompletedTask;

		return default;
	}
}
